# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

__all__ = ["ExternalAccountType"]

ExternalAccountType = Literal[
    "cash", "checking", "general_ledger", "loan", "non_resident", "other", "overdraft", "savings"
]
