# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

__all__ = ["TransactionDirection"]

TransactionDirection = Literal["credit", "debit"]
